<?php
/**
 * @file mod_roomReservationTimesliceListBox.inc
 * A list box that allows the user to add and delete timeslices
 * @author Roland Hieber (roland.hieber@wilhelm-gym.net)
 * @date 23.06.2008
 * 
 * Copyright © 2007 Roland Hieber
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

require_once("ctrl.inc");
require_once("mod_room-reservation/mod_roomReservationControl.inc");
require_once("mod_room-reservation/mod_roomReservationTimeslice.inc");

/*****************************************************************************/
/**
 * @page timeslicelistbox_actions Actions of a 
 * mod_roomReservationTimesliceListBox instance
 * @{
 * The following constants describe the actions that a
 * mod_roomReservationTimesliceListBox instance can handle. They are used in
 * processRequestVariables() to determine the action that should be done when
 * the control is shown.
 */
/** Show the control (default action) */
define("MOD_ROOM_RESERVATION_TLB_ACTION_SHOW", 0);
/** Add a timeslice */
define("MOD_ROOM_RESERVATION_TLB_ACTION_ADD", 1);
/** Delete a timeslice */
define("MOD_ROOM_RESERVATION_TLB_ACTION_DELETE", 2);
/** @} */

/** @todo document, add a delete confirmation */
class mod_roomReservationTimesliceListBox  extends mod_roomReservationControl {
  
  /** 
   * (constant) The action to be done (GET data).
   * See @ref timeslicelistbox_actions.
   */
  protected $cAction;
  /** (string) The beginning for a new timeslice (GET data) */
  protected $sNewBegin;
  /** (string) The ending for a new timeslice (GET data) */
  protected $sNewEnd;
  
  /***************************************************************************/
  /**
   * @name Constructor
   * @{
   * Constructor
   * @param $oCfg (reference to mod_roomReservationConfig) Reference to the
   *  configuration
   * @return mod_roomReservationTimesliceListBox
   */
  public function __construct(mod_roomReservationConfig &$oCfg) {
    parent::__construct($oCfg);
  }
 
  /***************************************************************************/
  /**
   * @}
   * @name Initialization
   * @{
   */

  /**
   * Process the REQUEST variables and preset the some variables
   * @return void
   */
  protected function processRequestVariables() {
    
    if(isset($_GET["mod_roomReservationTimesliceListBox"])) {
      // action
      if(isset($_GET["mod_roomReservationTimesliceListBox"]["action"])) {
        $ga = $_GET["mod_roomReservationTimesliceListBox"]["action"];
        $this->setAction((($ga == "add") ? MOD_ROOM_RESERVATION_TLB_ACTION_ADD : 
          (($ga == "delete") ? MOD_ROOM_RESERVATION_TLB_ACTION_DELETE :
          MOD_ROOM_RESERVATION_TLB_ACTION_SHOW)));
      }
      
      // new beginning and new ending
      $this->setNewBegin(isset(
        $_GET["mod_roomReservationTimesliceListBox"]["begin"]) ?
        $_GET["mod_roomReservationTimesliceListBox"]["begin"] : "");
      $this->setNewEnd(isset(
        $_GET["mod_roomReservationTimesliceListBox"]["end"]) ?
        $_GET["mod_roomReservationTimesliceListBox"]["end"] : "");
    }
    
    // perform the requested action
    if($this->getAction() == MOD_ROOM_RESERVATION_TLB_ACTION_ADD) {
      // add a timeslice to the configuration file
      
      $bErrors = false;
      
      // Note: we want to handle the timestamps in GMT format, hence the "+0000"
      if(strtotime($this->getNewBegin()." +0000") === false) {
        $this->asMessages[] = _c("room-reservation:The beginning time is ".
        	"invalid.");
        $bErrors = true;
      }
      if(strtotime($this->getNewEnd()." +0000") === false) {
        $this->asMessages[] = _c("room-reservation:The ending time is ".
        	"invalid.");
        $bErrors = true;
      }
      
      if(!$bErrors) {
        try {
          $this->oCfg->addTimeslice(new mod_roomReservationTimeslice(
            strtotime($this->getNewBegin()." +0000") % 86400, 
            strtotime($this->getNewEnd()." +0000") % 86400));
          $this->oCfg->writeConfig();
          $this->setNewBegin("");
          $this->setNewEnd("");
        } catch(Exception $e) {
          $this->asMessages[] = $e->getMessage();
        }
      }
      
    } elseif($this->getAction() == MOD_ROOM_RESERVATION_TLB_ACTION_DELETE) {
      // delete a timeslice from the configuration file
      if(isset($_POST["mod_roomReservationTimesliceListBox"])) {
        if(isset($_POST["mod_roomReservationTimesliceListBox"]["l"])) {
          $ao = $this->oCfg->getTimeslices();
          foreach($_POST["mod_roomReservationTimesliceListBox"]["l"] as 
            $n => $b) {
            if($b) {
              $this->oCfg->deleteTimeslice(new mod_roomReservationTimeslice(
                $ao[$n]->getBegin(), $ao[$n]->getEnd()));
            }
          }
          $this->oCfg->writeConfig();
        }
      }
    }
  }

  /***************************************************************************/
  /**
   * @}
   * @name Access to attributes
   * @{
   */
  
  /**
   * Set the action to be done (GET data)
   * @param $c (constant) Action. See @ref timeslicelistbox_actions.
   */
  protected function setAction($c) { $this->cAction = intval($c); }

  /**
   * Set the beginning for a new timeslice (GET data)
   * @param $s (string)
   */
  protected function setNewBegin($s) { $this->sNewBegin = $s; }
  
  /**
   * Set the beginning for a new timeslice (GET data)
   * @param $s (string)
   */
  protected function setNewEnd($s) { $this->sNewEnd = $s; }
  
  /**
   * Get the action to be done (GET data). See @ref timeslicelistbox_actions.
   * @return constant
   */
  function getAction() { return $this->cAction; }
  
  /**
   * Get the beginning for a new timeslice (GET data)
   * @return string
   */
  public function getNewBegin() { return $this->sNewBegin; }
  
  /**
   * Get the beginning for a new timeslice (GET data)
   * @return string
   */
  public function getNewEnd() { return $this->sNewEnd; }
  
  /***************************************************************************/
  /**
   * @}
   * @name Output
   * @{
   */
  
  /**
   * Actually show the control
   * @return void
   */
  public function doShow() {
    TreeView(array(_c("room-reservation:Begin"), _c("room-reservation:End")));

    // addition form
    printf("<form method='get'>");
    hidden("mod_roomReservationTimesliceListBox[action]", "add");
    TreeViewTitle(_("Add"));
    
    $sMessages = $this->getMessages();
    if(trim($sMessages) != "") {
      TreeViewLine($sMessages);
    }
    
    TreeViewLine(array(sprintf("<%s name='mod_roomReservationTimesliceListBox".
    	"[begin]' value='%s' size='8'/>", $GLOBALS["stdedt"], 
      $this->getNewBegin()), sprintf("<%s name='".
      "mod_roomReservationTimesliceListBox[end]' value='%s' size='8'/> <%s ".
      "name='mod_roomReservationTimesliceListBox[submit]' value='%s' />", 
      $GLOBALS["stdedt"], $this->getNewEnd(), $GLOBALS["stdbtn"], _("Add"))));
    echo "</form>\n";
    
    // deletion form
    TreeViewTitle(_c("room-reservation:Periods"));
    $aoTs = $this->oCfg->getTimeslices();
    if(count($aoTs) > 0) {
      echo "<form action='?mod_roomReservationTimesliceListBox[action]=delete' ".
      	"method='post'>";
      $i = 0;
      foreach($aoTs as $oTs) {
        $sBox = sprintf("<input type='hidden' ".
        	"name='mod_roomReservationTimesliceListBox[l][%d]' value='0' />".
        	"<%s name='mod_roomReservationTimesliceListBox[l][%d]' value='1' />",
          $i, $GLOBALS["smlchk"], $i);
        // Note: we have only GMT timestamps in the timeslice objects
        TreeViewLine(array($sBox . gmstrftime(_("%#I:%M %p"), $oTs->getBegin()),
          gmstrftime(_("%#I:%M %p"), $oTs->getEnd())));
        $i++;
      }
      // toolbar
      printf("<tr><td class='tbbtm' colspan='%d'>", $GLOBALS["treeview_cols"]);
      CheckCombo();
      printf("<%s name='mod_roomReservationTimesliceListBox[submit]' ".
      	"value='%s' />", $GLOBALS["stdbtn"], _("Delete"));
      echo "</td></tr>\n";
    } else {
      TreeViewEmpty();
    }
    
    echo "</form>\n";
    _TreeView();
  }
  /** @} */
}
?>